#-------------------------------- matutil.py --------------------------------
#  Contains utility routines for basic operations with vectors and matrices.
#  Author: Titus Beu, 2013
#----------------------------------------------------------------------------
from math import *

#============================================================================
def MatRead(a, n, m):
#----------------------------------------------------------------------------
#  Reads from the keyboard the elements of matrix a[1:n][1:m]
#----------------------------------------------------------------------------
   for i in range(1,n+1):
      for j in range (1,m+1):
         print("[",i,"][",j,"] = ",end=""); a[i][j] = input()

#============================================================================
def MatPrint(a, n, m):
#----------------------------------------------------------------------------
#  Prints the elements of matrix a[1:n][1:m] on the display
#----------------------------------------------------------------------------
   for i in range(1,n+1):
      for j in range (1,m+1): print('{0:11.2e}'.format(a[i][j]),end="")
      print()

#============================================================================
def MatPrintTrans(a, n, m):
#----------------------------------------------------------------------------
#  Prints the elements of transposed matrix a[1:n][1:m] on the display
#----------------------------------------------------------------------------
   for j in range(1,m+1):
      for i in range (1,n+1): print('{0:11.2e}'.format(a[i][j]),end="")
      print()

#============================================================================
def MatZero(a, n, m):
#----------------------------------------------------------------------------
#  Zeros the elements of matrix a[1:n][1:m]
#----------------------------------------------------------------------------
   for j in range(1,m+1):
      for i in range (1,n+1): a[i][j] = 0e0

#============================================================================
def MatCopy(a, b, n, m):
#----------------------------------------------------------------------------
#  Copies matrix a[1:n][1:m] into matrix b[1:n][1:m]
#----------------------------------------------------------------------------
   for i in range(1,n+1):
      for j in range (1,m+1): b[i][j] = a[i][j]

#============================================================================
def MatTrans(a, n):
#----------------------------------------------------------------------------
#  Replaces the square matrix a[1:n][1:n] by its transpose
#----------------------------------------------------------------------------
   for i in range(2,n+1):
      for j in range(1,i):
         t = a[i][j]; a[i][j] = a[j][i]; a[j][i] = t

#============================================================================
def MatDiff(a, b, c, n, m):
#----------------------------------------------------------------------------
#  Returns the difference of matrices a and b in c[1:n][1:m]
#----------------------------------------------------------------------------
   for i in range(1,n+1): 
      for j in range (1,m+1): c[i][j] = a[i][j] - b[i][j]

#============================================================================
def MatProd(a, b, c, n, l, m):
#----------------------------------------------------------------------------
#  Returns the product of matrices a[1:n][1:l] and b[1:l][1:m] in c[1:n][1:m]
#----------------------------------------------------------------------------
   for i in range(1,n+1):
      for j in range (1,m+1):
         t = 0e0
         for k in range(1,l+1): t += a[i][k] * b[k][j]
         c[i][j] = t

#============================================================================
def MatPow(m, a, b, n):
#----------------------------------------------------------------------------
#  Returns the m-th power of the square matrix a[1:n][1:n] in b[1:n][1:n]
#----------------------------------------------------------------------------
   c = [[0]*(n+1) for i in range(n+1)]                           # work array

   MatCopy(a,b,n,n)                                              # case m = 1

   for k in range(2,m+1):                           # repeated multiplication
      MatProd(a,b,c,n,n,n)
      MatCopy(c,b,n,n)

#============================================================================
def MatNorm(a, n, m):
#----------------------------------------------------------------------------
#  Returns the max norm of matrix a[1:n][1:m], i.e. max|a[i][j]|
#----------------------------------------------------------------------------
   norm = 0e0
   for i in range(1,n+1):
      for j in range(1,m+1):
         if (norm < fabs(a[i][j])): norm = fabs(a[i][j])
   return norm

#============================================================================
def VecPrint(a, n):
#----------------------------------------------------------------------------
#  Prints the elements of vector a[1:n] on the display
#----------------------------------------------------------------------------
   for i in range(1,n+1): print('{0:11.2e}'.format(a[i]),end="")
   print()

#============================================================================
def VecZero(a, n):
#----------------------------------------------------------------------------
#  Zeros the elements of vector a[1:n]
#----------------------------------------------------------------------------
   for i in range (1,n+1): a[i] = 0e0

#============================================================================
def VecCopy(a, b, n):
#----------------------------------------------------------------------------
#  Copies vector a[1:n] into vector b[1:n]
#----------------------------------------------------------------------------
   for i in range(1,n+1): b[i] = a[i]

#============================================================================
def VecDiff(a, b, c, n):
#----------------------------------------------------------------------------
#  Returns the difference of vectors a and b in c[1:n]
#----------------------------------------------------------------------------
   for i in range(1,n+1): c[i] = a[i] - b[i]

#============================================================================
def VecNorm(a, n):
#----------------------------------------------------------------------------
#  Returns the 2-norm of a vector a[1:n]
#----------------------------------------------------------------------------
   norm = 0e0
   for i in range(1,n+1): norm += a[i] * a[i]
   norm = sqrt(norm)
   return norm

#============================================================================
def MatVecProd(a, b, c, n):
#----------------------------------------------------------------------------
#  Returns the product of matrix a[1:n][1:n] and vector b[1:n] in c[1:n]
#----------------------------------------------------------------------------
   for i in range(1,n+1):
      t = 0e0
      for j in range (1,n+1): t += a[i][j] * b[j]
      c[i] = t
